# Generated by roxygen2: do not edit by hand

S3method(plot,babebi_fit)
S3method(print,babebi_fit)
S3method(print,babebi_validation)
S3method(print,summary.babebi_fit)
S3method(summary,babebi_fit)
export(babebi)
export(fit_model)
export(montecarlo_from_data)
