% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{plot.babebi_fit}
\alias{plot.babebi_fit}
\title{Plot a babebi fit object}
\usage{
\method{plot}{babebi_fit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{babebi_fit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
No return value, called for its side effects.
}
\description{
Plot a babebi fit object
}
