% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{print.babebi_validation}
\alias{print.babebi_validation}
\title{Print a babebi validation object}
\usage{
\method{print}{babebi_validation}(x, digits = 3, ...)
}
\arguments{
\item{x}{An object of class \code{babebi_validation}.}

\item{digits}{Number of digits to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print a babebi validation object
}
