% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{summary.babebi_fit}
\alias{summary.babebi_fit}
\title{Summarise a babebi fit object}
\usage{
\method{summary}{babebi_fit}(object, digits = 3, ...)
}
\arguments{
\item{object}{An object of class \code{babebi_fit}.}

\item{digits}{Number of digits to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{summary.babebi_fit}.
}
\description{
Summarise a babebi fit object
}
